/// @addtogroup bitSet
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file BmpHeader.h
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) Marian Mrozek 2005-2006
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.





#ifndef _BMP_HEADER_H
#define _BMP_HEADER_H

#include <sstream>
#include <stdexcept>
#include "capd/bitSet/EmbDimException.h"

template <typename word, int dim>
struct BmpHeader;

template<>
struct BmpHeader<unsigned long int,2>{
  static const unsigned int standardType='M'*256+'B'; // 19778
  static const unsigned int multidimType='D'*256+'B'; //
  unsigned int type;               // Magic identifier
  unsigned int size;
  unsigned int reserved1;
  unsigned int reserved2;
  unsigned int offset;             // Offset to image data in bytes
  unsigned int ihsize;             // Info header size in bytes
  int width;                       // Width of image
  int height;                      // Height of image
  unsigned int planes;             // Number of colour planes
  unsigned int bits;               // Bits per pixel
  unsigned int compression;        // Compression type
  unsigned int imagesize;          // Image size in bytes
  int xresolution;                 // Pixels per meter in x dir
  int yresolution;                 // Pixels per meter in y dir
  unsigned int ncolours;           // Number of colours
  unsigned int importantcolours;   // Important colours
  unsigned int color0;             // palette code for index 0
  unsigned int color1;             // palette code for index 1

  void read(std::fstream& file);
  void write(std::fstream& file);
  void showHeader();
};

template <typename word, int dim>
struct BmpHeader{
  static const unsigned int standardType='M'*256+'B'; // 19778
  static const unsigned int multidimType='D'*256+'B'; //
  static const unsigned int repsetType='R'*256+'B'; //
  unsigned int type;               // Magic identifier
  int dimension;
  int length;
  int actualDimZeroBitWidth;
  int paddedBitWidth[dim];
  void read(std::fstream& file);
  void write(std::fstream& file);
  void showHeader();
};


#endif

/// @}

