/// @addtogroup homologicalAlgebra
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file ChainAux.hpp
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#if !defined(_CHAINAUX_HPP_)
#define _CHAINAUX_HPP_
#include "capd/homologicalAlgebra/ChainAux.h"
/*
  For the moment we have here only the implementation of matrixColumnsToChains
*/

// -------------------------------------------------------------------------------------- //

// ********** matrixColumnsToChains ********** //
template<typename freeModuleType>
void matrixColumnsToChains(
  const freeModuleType& A_BaseModule,
  const typename freeModuleType::MatrixType& A_matrix,
  std::vector<ChainAux<freeModuleType> >& A_chains
){
  int m=A_matrix.numberOfRows();
  int n=A_matrix.numberOfColumns();
  A_chains.reserve(n);
  for(int j=0;j<n;++j){
    ChainAux<freeModuleType> c;
    c.m_pBaseModule=&A_BaseModule;
    for(int i=0;i<m;++i){
      typename freeModuleType::MatrixType::ScalarType zero(0);
      if( A_matrix[i][j] != zero ){
        c.m_chainCoef.insert(make_pair(i,A_matrix[i][j]));
      }
    }
    A_chains.push_back(c);
  }
}  // ********** matrixColumnsToChains ********** //
#endif //_CHAINAUX_HPP_

/// @}

