//////////////////////////////////////////////////////////////////////////////
//   Package:          CAPD
//   Subpackage:
/// @addtogroup intervals
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file intervalDeprecated.h
///
/// Interval Arithmetics package 
///
/// @author Tomasz Kapela   @date 11-01-2006
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) Tomasz Kapela 2006
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.


#ifndef _INTERVAL_DEPRECATED_H_
#define _INTERVAL_DEPRECATED_H_

#ifdef __INTERVAL_DEPRECATED__

namespace chomp{
namespace intervals{

template < typename T_Bound, typename T_Rnd> 
inline int in(T_Bound _value, const Interval< T_Bound, T_Rnd >& iv)  // inclusion
{
  return iv.contains(_value);
}

template < typename T_Bound, typename T_Rnd> 
inline int in(T_Bound _value, const Interval< T_Bound, T_Rnd >& iv, int interior)  // inclusion
{
  if(interior)
    return iv.containsInInterior(_value);
  else
    return iv.contains(_value);
}

template < typename T_Bound, typename T_Rnd> 
inline int in(const Interval< T_Bound, T_Rnd >& small_iv ,const Interval< T_Bound, T_Rnd >& iv, int interior)// inclusion
{
  if(interior)
    return small_iv.subsetInterior(iv);
  else
    return small_iv.subset(iv);
}


template < typename T_Bound, typename T_Rnd> 
inline Interval< T_Bound, T_Rnd > Left(const Interval< T_Bound, T_Rnd > &iv)
{
  return iv.left();
}
template < typename T_Bound, typename T_Rnd> 
inline Interval< T_Bound, T_Rnd > Right(const Interval< T_Bound, T_Rnd > &iv)
{
  return iv.right();
}

inline double Left(double d)
{
   return d;
}

inline double Right(double d)
{
   return d;
}

template < typename T_Bound, typename T_Rnd> 
inline Interval< T_Bound, T_Rnd > diameter(const Interval< T_Bound, T_Rnd > &iv)
{
  return diam(iv);
}

template < typename T_Bound, typename T_Rnd> 
inline void iSplit(Interval< T_Bound, T_Rnd > *iv, T_Bound *diam)
{
  split(*iv, *diam);
}

template < typename T_Bound, typename T_Rnd> 
inline int Degenerate(const Interval< T_Bound, T_Rnd > &iv)
{
   return iv.contains(0.0);
}

inline int Degenerate(double &r)
{
   return (r==0.0);
}

template < typename T_Bound, typename T_Rnd> 
inline double fabs(const Interval< T_Bound, T_Rnd > &iv)
{
   return chomp::abs(iv).rightBound();
}

template < typename T_Bound, typename T_Rnd> 
inline Interval< T_Bound, T_Rnd> Ihull( const Interval< T_Bound, T_Rnd> & i1, 
                                        const Interval< T_Bound, T_Rnd> & i2)
{
   return intervalHull(i1, i2);
}

template < typename T_Bound, typename T_Rnd> 
inline Interval< T_Bound, T_Rnd> solve_affine_inclusion(
                                                 const Interval< T_Bound, T_Rnd> & a, 
                                                 const Interval< T_Bound, T_Rnd> & p, 
                                                 const Interval< T_Bound, T_Rnd> & c)
{
   return solveAffineInclusion(a, p, c);
}

template < typename T_Bound, typename T_Rnd> 
inline Interval< T_Bound, T_Rnd> solve_affine_inclusion(
                                                 const Interval< T_Bound, T_Rnd> & a, 
                                                 const Interval< T_Bound, T_Rnd> & p, 
                                                 const Interval< T_Bound, T_Rnd> & c,
                                                 int & dir)
{
   return solveAffineInclusion(a, p, c, dir);
}

}} // namespace chomp::intervals 


#endif // __INTERVAL_DEPRECATED__

#endif //  _INTERVAL_DEPRECATED_H_

/// @}

