/////////////////////////////////////////////////////////////////////////////
//   Package:          CAPD
//   Subpackage: 
/// @addtogroup intervals
/// @{ 

/////////////////////////////////////////////////////////////////////////////
/// @file mpIntervalFun.h
///
/// @author Tomasz Kapela   @date 23-08-2006
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) Tomasz Kapela 2006 
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.


#ifndef _MP_INTERVAL_FUN_H_
#define _MP_INTERVAL_FUN_H_

#include <exception>
#include "capd/multiPrec/mpfrClass.h"
#include "capd/interval/mpIntervalSettings.h"

class NaNException: public std::exception{
  const char * what() const throw ()
  {
    return "Not a number. ";
  }
};


#ifdef __HAVE_MPFR__

namespace chomp{
namespace intervals{

typedef chomp::multiPrec::MpfrClass MpfrClass;


inline bool isinf(const Interval<MpfrClass> & r) 
{ 
  return (isinf(r.leftBound()) || isinf(r.rightBound())); 
}

inline void testNaN(const Interval<MpfrClass> & x)
{
#ifdef __MPI_TEST_NAN__
  if(isnan(x.leftBound()) || isnan(x.rightBound())))
    throw NaNException();  
#endif  // __MPI_TEST_NAN__
}


template<>
Interval<MpfrClass> sin(const Interval<MpfrClass>& x);

template<> 
Interval<MpfrClass> cos(const Interval<MpfrClass>& x);

template<>
Interval<MpfrClass> tan(const Interval<MpfrClass>& x);


template<>
Interval<MpfrClass> log(const Interval<MpfrClass>& x);

template<>
Interval<MpfrClass> exp(const Interval<MpfrClass>& x);

//-------- sqrt -----------------
template<>
Interval<MpfrClass> sqrt(const Interval<MpfrClass>  & z);

template<>
inline Interval<MpfrClass> power(const Interval<MpfrClass> &a, const Interval<MpfrClass> &b)
{
  if( a.leftBound() < 0 )
  {
    throw std::range_error("power(A, B): interval A must be greater than zero\n");
  }
  return exp(b * log(a));
}

template<>
Interval<MpfrClass> power(const Interval<MpfrClass>& value, /*long*/  int exponent);

template<>
inline Interval<MpfrClass>  Interval<MpfrClass>::pi()
{
  
  chomp::multiPrec::PrecisionType prec = BoundType::getDefaultPrecision();
  return Interval<MpfrClass>(BoundType::Pi(chomp::multiPrec::RoundDown, prec), BoundType::Pi(chomp::multiPrec::RoundUp, prec));
}

/*
//  So far not defined in Interval template 
//
template<>
Interval<MpfrClass> Interval<MpfrClass>::log2()
{

   chomp::multiPrec::PrecisionType   prec = BoundType::getDefaultPrecision();
   return Interval<MpfrClass>(BoundType::Log2(chomp::multiPrec::RoundDown, prec), BoundType::Log2(chomp::multiPrec::RoundUp, prec));
}
*/

template<>
inline Interval<MpfrClass> Interval<MpfrClass>::euler()
{
   chomp::multiPrec::PrecisionType   prec = BoundType::getDefaultPrecision();
   return Interval<MpfrClass>(BoundType::Euler(chomp::multiPrec::RoundDown, prec), BoundType::Euler(chomp::multiPrec::RoundUp, prec));
}


}}  // end of namespace chomp::intervals

#endif  // __HAVE_MPFR__

#endif  // _MP_INTERVAL_FUN_

/// @}

