/// @addtogroup krak
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file atom.h
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _atom_h_
#  define _atom_h_

#  include <iostream>
#  include <fstream>

#define NONE  -1
#define Atom   0
#define STRING 1
#define LINK   2
#define HLINK  3
#define LIST   4
#define RECORD 5
#define ITEM   6
#define VAR    7
#define FIELD  8
#define CONST_STRING 9

namespace chomp{
namespace krak{
   class frstring;
   void errorExit(const char *,...);
}}

// class atom is the parent class for classes frstring, item, var, list and record

namespace chomp{
namespace krak{

class atom {
public:
   int virtual who(void) = 0 ;
   virtual ~atom(){}
   virtual atom *copy(void) = 0 ;
   virtual chomp::krak::frstring descriptor(void) = 0 ;
   virtual void put(std::ostream &s){}
   int virtual operator<=(atom &a2);
   int virtual operator>=(atom &a2);
   int virtual operator==(atom &a2);
};//end class atom
}} // the end of the namespace chomp::krak

inline int chomp::krak::atom::operator<=(chomp::krak::atom &a2)
{
   chomp::krak::errorExit("Do not call functions from class atom");
   return 1;
}

inline int chomp::krak::atom::operator>=(chomp::krak::atom &a2)
{
   chomp::krak::errorExit("Do not call functions from class atom");
   return 1;
}

inline int chomp::krak::atom::operator==(chomp::krak::atom &a2)
{
   chomp::krak::errorExit("Do not call functions from class atom");
   return 1;
}

#endif

/// @}
