/// @addtogroup krak
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file item.h
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _item_h_
#define _item_h_
#include "capd/krak/chain.h"

/************************************ item **********************************/
namespace chomp{
namespace krak{
class item : public chomp::krak::link, public virtual chomp::krak::frstring
/*  Linkable free strings
*/
{
   friend class list;
protected:

  // constructors
   item(void) : chomp::krak::frstring(), chomp::krak::link(){}
   item(item &another_item) : chomp::krak::frstring(another_item), chomp::krak::link() {}
   item(const chomp::krak::frstring &from) : chomp::krak::frstring(from), chomp::krak::link() {}
   item(char *init_string) : chomp::krak::frstring(init_string), chomp::krak::link() {}

   virtual chomp::krak::atom *copy(void);    //virtual constructor
   virtual ~item(void);         //destructor

public:

   int virtual who(void);                //test what atom (see krak/atom.h)
   int virtual operator<=(chomp::krak::atom &a2);
   int virtual operator>=(chomp::krak::atom &a2);
   int virtual operator==(chomp::krak::atom &a2);
   chomp::krak::frstring virtual descriptor(void);
   void put(std::ostream &out);
};

/************************************ var **********************************/

class var : public virtual chomp::krak::frstring
/* variables, have name inherited form string and values,
   which may be arbitraty atoms.
*/
{
public:
   chomp::krak::atom *value;            // pointer to value, may be NULL

//constructors
   var(void);
   var(var &a_var);
   var(char *text);
   var(chomp::krak::frstring &text);
   var(chomp::krak::frstring &s, chomp::krak::atom &a);
   var(chomp::krak::frstring &s, chomp::krak::frstring &v);

protected:
   virtual chomp::krak::atom *copy();            //virtual constructor
   virtual ~var(void);              //destructor

public:
   var &operator=(var &a_var);          //takes over the name and value
   frstring virtual descriptor(void);
   var &operator|=(chomp::krak::atom &a);            //assign copy of *atom to value
   var &operator|=(chomp::krak::frstring &s);        //assign copy of *s to value
   chomp::krak::frstring &operator!(void);           //return value if frstring (???)
                                       //will fail if value is not an item
   chomp::krak::atom *operator~(void);               //return pointer to value
   int virtual who(void);               //test what atom (see krak/atom.h)
   void virtual put(std::ostream &out);
};

/************************************ field **********************************/

class field : public item, public var
/*  field is a linkable var;
*/
{
   friend class record;
protected:

  //constructors
   field(void);
   field(chomp::krak::frstring &t);                  //a field with no value
   field(chomp::krak::frstring &t,chomp::krak::atom &a);
   field(chomp::krak::frstring &t,chomp::krak::frstring &s);

   virtual ~field(void);               //destructor
   virtual chomp::krak::atom *copy();               //virtual constructor
   field &operator=(field &a_field);   //copies everything, including followers
                                      //probably not save in usage !!!!!
public:

   int virtual who(void);              //test what atom (see krak/atom.h)
   chomp::krak::frstring virtual descriptor(void);
   void virtual put(std::ostream &out);
};

/******************************* pointing_field ****************************/

class pointing_field : public field
/* introduced to enable alternatively sorted records
   however its usage not clear. Even not clear if used at all !!!
*/
{
public:

   pointing_field(void);
   virtual ~pointing_field(void);
};

/************************************ list **********************************/

class list : public hlink
/*  stores a linked list of atoms;
*/
{
public:

   list(void);                           //constructor
   int virtual who(void);                //test what atom (see krak/atom.h)
   virtual chomp::krak::atom *copy(void);             //virtual constructor
   chomp::krak::frstring virtual descriptor(void);
   list &operator+(chomp::krak::frstring s);         //insert the given string at the end
   void additem(chomp::krak::frstring &s);            //the same, different syntax
   chomp::krak::atom *operator[](int i);              //pointer to the element at the given
                                        //position
   virtual void put(std::ostream &out);
};

/************************************ record **********************************/

class record : public list
{
public:

   record(void);                         //constructor
   int virtual who(void);                //test what atom (see krak/atom.h)
   virtual chomp::krak::atom *copy(void);             //virtual constructor
   chomp::krak::frstring virtual descriptor(void);
   record &operator+(chomp::krak::frstring s);       //insert a field of a give name,
                                        // but no value
// various version of adding a field with value
   void addfield(chomp::krak::frstring &name, chomp::krak::atom &val);
   void addfield(chomp::krak::frstring &name, chomp::krak::frstring &val);
   void addfield(chomp::krak::frstring &name, chomp::krak::const_string &val);
   void addpointing_field(chomp::krak::frstring &name, chomp::krak::atom &val);

// copy the fields with their names but assign NULL to values
   record *framecopy(void);
// copy the fields with their names,
//  preserve the type of values but reassign them to trivial copies
//  descent recursively other records
   record *novalcopy(void);

//return pointer to the field of the given name
   chomp::krak::var &operator[](chomp::krak::frstring &s);
// search for the first field in the record which has non trivial value,
// treat it as a master copy and adjust all other fields to have
// the same structure.

   void update(void);

   virtual void put(std::ostream &out);
};

}} // the end of the namespace chomp::krak

//########################### inline definitions ##############################/

/************************************ item **********************************/

inline chomp::krak::atom *chomp::krak::item::copy(void)
{
   chomp::krak::item *i = new chomp::krak::item;
   chomp::krak::atom *a=(chomp::krak::atom *)(void *)i;
   chomp::krak::frstring *s=i;
   *s=*this;
   i->nxt=( next()==NULL ? NULL : (chomp::krak::link *)(next()->copy()) );
   return a;
}

/*
inline chomp::krak::atom *chomp::krak::item::copy(void)
{
   chomp::krak::item *i = new chomp::krak::item;
   chomp::krak::frstring *s=i;
   *s=*this;
   i->nxt=( next()==NULL ? NULL : (chomp::krak::link *)(next()->copy()) );
   return (chomp::krak::link *)i;
}
*/

inline chomp::krak::item::~item(void)
{
   delete nxt;
   nxt=NULL;
   delete str;
   str=NULL;
}

inline int chomp::krak::item::who(void)
{
   return ITEM;
}

inline int chomp::krak::item::operator<=(chomp::krak::atom &a2)
{
   chomp::krak::item *item2=(chomp::krak::item *)(void *)&a2;
   chomp::krak::frstring *s1=this,*s2=item2;
   return *s1<=*s2;
}

inline int chomp::krak::item::operator>=(chomp::krak::atom &a2)
{
   chomp::krak::item *item2=(chomp::krak::item *)(void *)&a2;
   chomp::krak::frstring *s1=this,*s2=item2;
   return *s1>=*s2;
}

inline int chomp::krak::item::operator==(chomp::krak::atom &a2)
{
   chomp::krak::item *item2 = (chomp::krak::item *)(void *)&a2;
   chomp::krak::frstring *s1=this,*s2=item2;
   return *s1==*s2;
}

inline chomp::krak::frstring chomp::krak::item::descriptor(void)
{
   chomp::krak::frstring s="Item: "^*this^chomp::krak::frstring("\n");
   if (nxt!=NULL) s=s^nxt->descriptor();
   return s;
}

inline void chomp::krak::item::put(std::ostream &out)
{
   chomp::krak::frstring::put(out);
   out.put('\n');
   if (next()!=NULL) next()->put(out);
}

/************************************** var ***********************************/

inline chomp::krak::var::var(void)
{
   value=NULL;
}

inline chomp::krak::var::var(chomp::krak::var &a_var) : chomp::krak::frstring(a_var)
{
   value=(a_var.value==NULL ? NULL : a_var.copy());
}

inline chomp::krak::var::var(char *text): chomp::krak::frstring(text)
{
   value=NULL;
}

inline chomp::krak::var::var(chomp::krak::frstring &text): chomp::krak::frstring(text)
{
   value=NULL;
}

inline chomp::krak::var::var(chomp::krak::frstring &s, chomp::krak::atom &a) : chomp::krak::frstring(s)
{
   value=a.copy();
}

inline chomp::krak::var::var(chomp::krak::frstring &s, chomp::krak::frstring &v) : chomp::krak::frstring(s)
{
   value=v.copy();
}

inline chomp::krak::atom *chomp::krak::var::copy()
{
   chomp::krak::var *v = new chomp::krak::var;
   chomp::krak::frstring *s=v;
   *s=*this;
   v->value=(value==NULL ? NULL : value->copy());
   return v;
}

inline chomp::krak::var::~var(void)
{
   delete str;
   str=NULL;
   delete value;
   value=NULL;
}

inline chomp::krak::var &chomp::krak::var::operator=(chomp::krak::var &a_var)
{
   this->~var();
   (*this).frstring::operator=(a_var);
   value=(a_var.value==NULL ? NULL : a_var.value->copy());
   return *this;
}

inline chomp::krak::frstring chomp::krak::var::descriptor(void)
{
   chomp::krak::frstring t;
   t="Variable: "^*this^chomp::krak::frstring("\n")^chomp::krak::frstring("With value=[");

   if(value==NULL) t=t^"NULL\n";
   else t=t^value->descriptor();
   t=t^"]\n";
   return t;
}


inline chomp::krak::var &chomp::krak::var::operator|=(chomp::krak::atom &a)
{
   value=a.copy();
   return *this;
}

inline chomp::krak::var &chomp::krak::var::operator|=(chomp::krak::frstring &s)
{
   value=s.copy();
   return *this;
}

inline chomp::krak::frstring &chomp::krak::var::operator!(void)
{
   return *((chomp::krak::frstring *)value);
}

inline chomp::krak::atom *chomp::krak::var::operator~(void)
{
   return value;
}

inline int chomp::krak::var::who(void)
{
   return VAR;
}

inline void chomp::krak::var::put(std::ostream &out)
{
   chomp::krak::frstring::put(out);
   out.put(':');
   if(value!=NULL) value->put(out);
   out.put('\n');
}

/********************************** field ************************************/

inline chomp::krak::field::field(void)
   : chomp::krak::frstring(), chomp::krak::item(), chomp::krak::var() {}

inline chomp::krak::field::field(chomp::krak::frstring &t)
   : chomp::krak::frstring(t), chomp::krak::item(), chomp::krak::var() {}

inline chomp::krak::field::field(chomp::krak::frstring &t, chomp::krak::atom &a)
   : chomp::krak::frstring(t), chomp::krak::var(t,a){}

inline chomp::krak::field::field(chomp::krak::frstring &t,chomp::krak::frstring &s)
   : chomp::krak::frstring(t), chomp::krak::var(t,s){}

inline chomp::krak::field::~field(void)
{
   delete str;
   delete value;
   delete nxt;
   str=NULL;value=NULL;nxt=NULL;
}


inline chomp::krak::atom *chomp::krak::field::copy()
{
   chomp::krak::field *f = new chomp::krak::field;
   chomp::krak::atom *a = (chomp::krak::atom *)(void *)f;
   chomp::krak::frstring *s=f;
   *s=*this;
   f->nxt=(nxt==NULL ? NULL : (chomp::krak::link *)(nxt->copy()));
   f->value=(value==NULL ? NULL : value->copy());
   return a;
}

/*
inline chomp::krak::atom *chomp::krak::field::copy()
{
   chomp::krak::field *f=new chomp::krak::field;
   chomp::krak::frstring *s=f;
   *s=*this;
   f->nxt=(nxt==NULL ? NULL : (chomp::krak::link *)(nxt->copy()));
   f->value=(value==NULL ? NULL : value->copy());
   return (chomp::krak::link *)f;
}
*/

inline chomp::krak::field &chomp::krak::field::operator=(chomp::krak::field &a_field)
{
   this->~field();
   (*this).frstring::operator=(a_field);
   value=(a_field.value==NULL ? NULL : a_field.value->copy());
   nxt=(a_field.nxt==NULL ? NULL : (chomp::krak::link *)(a_field.nxt->copy()));
   return *this;
}

inline int chomp::krak::field::who(void)
{
   return FIELD;
}

inline chomp::krak::frstring chomp::krak::field::descriptor(void)
{
   chomp::krak::frstring t;
   t="Field : "^*this^chomp::krak::frstring("\n")^chomp::krak::frstring("With value=[");
   if(value==NULL) t=t^"NULL\n";
   else t=t^value->descriptor();
   t=t^"]\n";
   if (nxt!=NULL) t=t^nxt->descriptor();
   return t;
}

inline void chomp::krak::field::put(std::ostream &out)
{
   chomp::krak::frstring::put(out);
   out.put(':');
   if(value!=NULL) value->put(out);
   out.put('\n');
   if(next()!=NULL) next()->put(out);
}

/******************************* pointing_field ****************************/

inline chomp::krak::pointing_field::pointing_field(void): chomp::krak::field(){}

inline chomp::krak::pointing_field::~pointing_field(void)
{
   delete str;
   delete nxt;
   str=NULL;
   value=NULL;
   nxt=NULL;
}

/************************************ list **********************************/

inline chomp::krak::list::list(void): chomp::krak::hlink(){}

inline int chomp::krak::list::who(void)
{
   return LIST;
}

inline chomp::krak::atom *chomp::krak::list::copy(void)
{
   chomp::krak::list *l = new chomp::krak::list;
   if(next()!=NULL) l->nxt=(chomp::krak::link *)(next()->copy());
   l->bind();
   return l;
}

inline chomp::krak::frstring chomp::krak::list::descriptor(void)
{
   chomp::krak::frstring s="A list which contains: {\n";
   if (next()!=NULL) s=s^next()->descriptor();
   s=s^"}\n";
   return s;
}

inline chomp::krak::list &chomp::krak::list::operator+(chomp::krak::frstring s)
{
   chomp::krak::hlink &h=*this;

   chomp::krak::item & an_item = * new chomp::krak::item(s);

   h+an_item;
//  hlink::operator+(an_item);
   return *this;
}

inline void chomp::krak::list::additem(chomp::krak::frstring &s)
{
   chomp::krak::hlink &h=*this;

   chomp::krak::item &an_item = *new chomp::krak::item(s);

   h.putlast(an_item);
//  putlast(an_item);
}

inline chomp::krak::atom *chomp::krak::list::operator[](int i)
{
   chomp::krak::link *l=(*this)>>i;
   return (chomp::krak::atom *)l;
}

inline void chomp::krak::list::put(std::ostream &out)
{
   out.put('{');
   out.put('\n');
   if(next()!=NULL) next()->put(out);
   out.put('}');
   out.put('\n');
}


/************************************ record **********************************/

inline chomp::krak::record::record(void): chomp::krak::list(){}

inline int chomp::krak::record::who(void)
{
   return RECORD;
}

inline chomp::krak::atom *chomp::krak::record::copy(void)
{
   chomp::krak::record *l = new chomp::krak::record;
   if(next()!=NULL) l->nxt=(chomp::krak::link *)(next()->copy());
   l->bind();
   return l;
}

inline chomp::krak::frstring chomp::krak::record::descriptor(void)
{
   chomp::krak::frstring s="A record which contains: {\n";
   if (next()!=NULL) s=s^next()->descriptor();
   s=s^"}\n";
   return s;
}

inline chomp::krak::record &chomp::krak::record::operator+(chomp::krak::frstring s)
{
   chomp::krak::hlink &h = *this;
   h+*new chomp::krak::field(s);
   return *this;
}

inline void chomp::krak::record::addfield(chomp::krak::frstring &name, chomp::krak::atom &val)
{
   chomp::krak::field &nf = *new chomp::krak::field;
   if (&nf==NULL) chomp::krak::errorExit("No memory for a new field");
   nf.frstring::operator=(name);
   nf.value=&val;
   putlast(nf);
}

inline void chomp::krak::record::addpointing_field(chomp::krak::frstring &name, chomp::krak::atom &val)
{
   chomp::krak::field &nf = *new chomp::krak::pointing_field;
   if (&nf==NULL) chomp::krak::errorExit("No memory for a new field");
   nf.frstring::operator=(name);
   nf.value=&val;
   putlast(nf);
}

inline void chomp::krak::record::addfield(chomp::krak::frstring &name, chomp::krak::frstring &val)
{
   chomp::krak::field &nf = *new chomp::krak::field;
   if (&nf==NULL) chomp::krak::errorExit("No memory for a new field");
   nf.frstring::operator=(name);
   nf.value = new chomp::krak::frstring(val);
   if(nf.value==NULL) chomp::krak::errorExit("No memory for a new frstring value");
   putlast(nf);
}

inline void chomp::krak::record::addfield(chomp::krak::frstring &name, chomp::krak::const_string &val)
{
   chomp::krak::field &nf = *new chomp::krak::field;
   if (&nf==NULL) chomp::krak::errorExit("No memory for a new field");
   nf.frstring::operator=(name);
   nf.value = new chomp::krak::const_string(val);
   if(nf.value==NULL) chomp::krak::errorExit("No memory for a new frstring value");
   putlast(nf);
}

inline void chomp::krak::record::put(std::ostream &out)
{
   out.put('[');
   out.put('\n');
   if(next()!=NULL) next()->put(out);
   out.put(']');
   out.put('\n');
}

  /******************************************************************************/

#endif

/// @}
