/// @addtogroup krak
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file job.h
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _job_h_
#define _job_h_

#include "capd/krak/frame.h"
#include "capd/krak/item.h"

#  define A_JOB       0
#  define BASIC_JOB   1
#  define SEL_JOB     2
#  define MENU_JOB    3
#  define VIEW_JOB    4
#  define DEL_INS_JOB 5
#  define ED_REC_JOB  6

namespace chomp{
namespace krak{
   enum Status{sleeping,running,finished};
   typedef int Stat;
   extern void (*restore_window)(void);
   class view_job;
   int confirm(chomp::krak::Frame &jobfrm, const char *prompt, const char *yes, const char *no);
}}


//############################  class job #################################
namespace chomp{
namespace krak{
class job : public chomp::krak::link
{
public:

   static job* runningJob;
   static job *rootJob,*requestedJob,*previousJob;
   static int desktopColor;

   static job* userSelectedJob(chomp::krak::UserMove &um);
   static chomp::krak::frstring message;
   static void mainLoop(void);

   chomp::krak::Frame jobfrm;
   chomp::krak::Stat status;
   int visible;
   job *parent;
   chomp::krak::hlink children;
   int order_no;
   int immortal;
   int request_repaint;     //introduced experimentally, so far use only in view_job;

   void makeInvisible(void);
   void makeVisible(void);
   virtual void furnish(void);
   job(void);
   job(const chomp::krak::Frame &prntfrm, const chomp::krak::At &lt, const chomp::krak::At &rb);
   job(int mylti, int myltj, int myrbi, int myrbj);
   job(const chomp::krak::Frame &frame);

   virtual int type();
   job* selectJob(chomp::krak::UserMove &um);
   virtual void launch(job *prnt=rootJob);
   virtual chomp::krak::Stat process(chomp::krak::UserMove &um);
   virtual void terminate(void);
   virtual void own_repaint(void);
   virtual void repaint(void);
   int exists(job *jobPt);
   int exists(int type);
};

//############################  class basic_job ################################

class basic_job : public job
{
   public:


   chomp::krak::frstring title;
   chomp::krak::Frame titlefrm,stopfrm,userfrm,helpfrm;
   chomp::krak::view_job *help;

   virtual void furnish(void);
   basic_job(void);

   basic_job(const chomp::krak::Frame &prntfrm,
      const chomp::krak::At &lt, const chomp::krak::At &rb, const chomp::krak::frstring &text);
   basic_job(int mylti, int myltj, int myrbi, int myrbj, const chomp::krak::frstring &text);
   basic_job(const chomp::krak::Frame &frame, const chomp::krak::frstring &text);
   virtual int type();
   virtual Stat process(chomp::krak::UserMove &um);
   virtual void own_repaint(void);
};


//############################  class sel_job #################################

class sel_job : public basic_job
{
public:
   chomp::krak::list &a_list;
   chomp::krak::Frame window,upFrm,downFrm;
   int selector,firstshown,lastshown,maxshown,length,search_token_length,search_permitted;
   chomp::krak::frstring search_token;
   int noStopFrm;

   virtual void launch(job *prnt=rootJob);
   virtual void furnish(void);

   sel_job(const chomp::krak::frstring &t,chomp::krak::list &l, const chomp::krak::Frame &f, int nostopfrm=0);
   virtual int type();
   virtual chomp::krak::Stat process(chomp::krak::UserMove &um);
   virtual void own_repaint(void);
   virtual int MakeSelection(chomp::krak::UserMove &um);
   void testUpDown(chomp::krak::UserMove &um);
};

//############################  class menu_job #################################

class menu_job  : public sel_job
{
public:
   void (*executor)(chomp::krak::frstring &);

   menu_job(const chomp::krak::frstring &t,list &l,const chomp::krak::Frame &f,
      void (*e)(chomp::krak::frstring &), int nostopfrm=0);
   virtual int type();
   virtual chomp::krak::Stat process(chomp::krak::UserMove &um);
};

//############################  class view_job #################################

class view_job  : public sel_job
{
public:
   chomp::krak::frstring bufor;
   int pos;               // current position in line;
   int field_width;       // seems to be unused
   int active,opened,size,with_file;
   chomp::krak::frstring filename;
   std::fstream job_file;

   view_job(const chomp::krak::frstring &t,chomp::krak::list &l,
      const chomp::krak::Frame &f,int act=1, int nostopfrm=0);
   view_job(const chomp::krak::frstring &fname,const chomp::krak::frstring &t,
      chomp::krak::list &l,const chomp::krak::Frame &f,int act=1);
   virtual int type();
   virtual chomp::krak::Stat process(chomp::krak::UserMove &um);
   virtual void own_repaint(void);
   void open(int read=0);
   void reset(void);                    //clear window
   void close(void);
   void width(int n);
//  friend view_job &operator<<(view_job &v, char *s);     //send string to window
   friend view_job &operator<<(view_job &v,chomp::krak::frstring f);   //send frstring to window
   friend view_job &operator<<(view_job &v, chomp::krak::Tab &t);        //send Tab to window
};

//############################  class del_ins_job ##############################

class del_ins_job : public sel_job
{
public:
   chomp::krak::Frame insfrm,delfrm;

   virtual void furnish(void);
   del_ins_job(const chomp::krak::frstring &t,chomp::krak::list &l,const chomp::krak::Frame &f);
   virtual int type();
   virtual chomp::krak::Stat process(chomp::krak::UserMove &um);
   virtual void own_repaint(void);
};

//############################  class ed_rec_job ###############################

class ed_rec_job : public sel_job
{
public:
   chomp::krak::Frame editfrm;
   int edit_field_start,edit_field_length;

   virtual void furnish(void);
   ed_rec_job(const chomp::krak::frstring &t,chomp::krak::record &r,const chomp::krak::Frame &f);
   virtual int type();
   virtual chomp::krak::Stat process(chomp::krak::UserMove &um);
   virtual void own_repaint(void);
   void set_edit_field_start(int percent); //set the proportion between the name
                                          //and value subframes
};

}} // the end of the namespace chomp::krak

//#############################################################################
//############################### inline definitions ##########################
//#############################################################################
namespace chomp{
namespace krak{
   inline void job_restore_window(void)
   {
      chomp::krak::job::rootJob->repaint();
   }
}}

//############################  class job #################################

inline void chomp::krak::job::makeInvisible(void)
{
   visible=0;
}

inline void chomp::krak::job::makeVisible(void)
{
   visible=1;
}

inline void chomp::krak::job::furnish(void){}

inline chomp::krak::job::job(const chomp::krak::Frame &prntfrm, const chomp::krak::At &lt, const chomp::krak::At &rb)
   : jobfrm(prntfrm,lt,rb,PINE,WHITE,0,0), status(sleeping), visible(1), parent(NULL),
   immortal(0), request_repaint(1)
{}

inline chomp::krak::job::job(int mylti, int myltj, int myrbi, int myrbj)
   : jobfrm(mylti,myltj,myrbi,myrbj,WHITE,BLACK,0,0), status(sleeping), visible(1), parent(NULL),
   immortal(0), request_repaint(1)
{}

inline chomp::krak::job::job(const chomp::krak::Frame &frame)
   : status(sleeping), visible(1), parent(NULL), immortal(0), request_repaint(1)
{
   jobfrm=frame;
   jobfrm.imarg=jobfrm.jmarg=0;
}

inline int chomp::krak::job::type()
{
   return A_JOB;
}

inline chomp::krak::Stat chomp::krak::job::process(chomp::krak::UserMove &um)
{
   return running;
}

inline void chomp::krak::job::own_repaint(void) {}

//############################### basic_job #################################

inline void chomp::krak::basic_job::furnish(void)
{
   jobfrm.adjust();
   titlefrm = chomp::krak::Frame(jobfrm,At(1,1),At(2,jobfrm.lCol-3),jobfrm.bgColor,jobfrm.fgColor);
   stopfrm = chomp::krak::Frame(jobfrm, chomp::krak::At(1,jobfrm.lCol-2),
      chomp::krak::At(2,jobfrm.lCol-1), jobfrm.bgColor, jobfrm.fgColor);
   helpfrm = chomp::krak::Frame(jobfrm, chomp::krak::At(1,jobfrm.lCol-4),
      chomp::krak::At(2,jobfrm.lCol-3), jobfrm.bgColor, jobfrm.fgColor);
   help=NULL;
}

inline chomp::krak::basic_job::basic_job(void)
{
   furnish();
}


inline chomp::krak::basic_job::basic_job(const chomp::krak::Frame &prntfrm,
   const chomp::krak::At &lt, const chomp::krak::At &rb, const chomp::krak::frstring &text)
   : chomp::krak::job(prntfrm,lt,rb)
{
   furnish();
   title=text;
}

inline chomp::krak::basic_job::basic_job(int mylti, int myltj, int myrbi, int myrbj,
   const chomp::krak::frstring &text) : chomp::krak::job(mylti,myltj,myrbi,myrbj)
{
   furnish();
   title=text;
}

inline chomp::krak::basic_job::basic_job(const chomp::krak::Frame &frame, const chomp::krak::frstring &text)
   : chomp::krak::job(frame)
{
   title=text;
   furnish();
}

inline int chomp::krak::basic_job::type()
{
   return BASIC_JOB;
}

//############################  class sel_job #################################

inline void chomp::krak::sel_job::furnish(void)
{
//    window=Frame(jobfrm,At(2,1),At(jobfrm.lRow-1,jobfrm.lCol-1),WHITE,BLACK);
   window = chomp::krak::Frame(jobfrm,At(3,1),At(jobfrm.lRow-1,jobfrm.lCol-2),WHITE,BLACK);
   upFrm = chomp::krak::Frame(jobfrm, chomp::krak::At(3,jobfrm.lCol-2),
      chomp::krak::At(jobfrm.lRow/2,jobfrm.lCol), jobfrm.bgColor, jobfrm.fgColor);
   downFrm = chomp::krak::Frame(jobfrm, chomp::krak::At(jobfrm.lRow/2+1,jobfrm.lCol-2),
      chomp::krak::At(jobfrm.lRow-1,jobfrm.lCol), jobfrm.bgColor, jobfrm.fgColor);
   maxshown=window.NoRow();
}

inline chomp::krak::sel_job::sel_job(const chomp::krak::frstring &t,
   chomp::krak::list &l, const chomp::krak::Frame &f, int nostopfrm)
   : chomp::krak::basic_job(f,t), a_list(l)
{
   furnish();
   search_token="";
   search_token_length=selector=firstshown=0;
   length=l.len();
   lastshown = chomp::min(length,maxshown)-1;
   search_permitted=0;
   noStopFrm=nostopfrm;
}

inline int chomp::krak::sel_job::type()
{
   return SEL_JOB;
}

//############################  class menu_job #################################

inline chomp::krak::menu_job::menu_job(const chomp::krak::frstring &t,
   chomp::krak::list &l,const chomp::krak::Frame &f, void (*e)(chomp::krak::frstring &), int nostopfrm)
   : chomp::krak::sel_job(t,l,f,nostopfrm)
{
   noStopFrm=nostopfrm;
   executor=e;
   search_permitted=1;
}

inline int chomp::krak::menu_job::type()
{
   return MENU_JOB;
}

//############################  class view_job #################################

inline chomp::krak::view_job::view_job(const chomp::krak::frstring &t,
   chomp::krak::list &l, const chomp::krak::Frame &f,int act,int nostopfrm)
   : chomp::krak::sel_job(t,l,f,nostopfrm), pos(0),
   field_width(0), active(act), opened(0), size(0), with_file(0)
{
     if(active) opened=1;
}

inline chomp::krak::view_job::view_job(const chomp::krak::frstring &fname,
   const chomp::krak::frstring &t, chomp::krak::list &l, const chomp::krak::Frame &f,int act)
   : chomp::krak::sel_job(t,l,f), pos(0),
   field_width(0), active(act), opened(0), size(0), with_file(1), filename(fname)
{
   if(active)opened=1;
}

inline int chomp::krak::view_job::type()
{
   return VIEW_JOB;
}

/*
  inline view_job &view_job::operator<<(frstring &s)
  begin
    return operator<<(t.string());
  end;
*/

inline void chomp::krak::view_job::reset(void)
{
   if(!with_file)
   {
      a_list.empty();
      search_token_length=selector=firstshown=length=0;
      lastshown = chomp::min(length,maxshown)-1;
      window.Clear();
      window << At(0,0);
   }
}

inline void chomp::krak::view_job::close(void)
{
   *this << "\n";
   if(!active)
   {
      firstshown=selector=0;
      length=a_list.len();
      lastshown = chomp::min(length,maxshown)-1;
      own_repaint();
   }
   if(with_file) job_file.close();
   opened=0;
}

inline void chomp::krak::view_job::width(int n)
{
   field_width=n;
}

//############################  class del_ins_job ##############################

inline void chomp::krak::del_ins_job::furnish(void)
{
   window = chomp::krak::Frame(jobfrm, chomp::krak::At(3,1),
      chomp::krak::At(jobfrm.lRow-5,jobfrm.lCol-2),WHITE,BLACK);
   maxshown=window.NoRow();
   insfrm = chomp::krak::Frame(jobfrm, chomp::krak::At(jobfrm.lRow-2,5),
      chomp::krak::At(jobfrm.lRow-1,8),jobfrm.bgColor,WHITE);
   delfrm = chomp::krak::Frame(jobfrm, chomp::krak::At(jobfrm.lRow-2,1),
      chomp::krak::At(jobfrm.lRow-1  ,4), jobfrm.bgColor,WHITE);
   upFrm = chomp::krak::Frame(jobfrm, chomp::krak::At(3,jobfrm.lCol-2),
      chomp::krak::At(jobfrm.lRow/2, jobfrm.lCol), jobfrm.bgColor, jobfrm.fgColor);
   downFrm = chomp::krak::Frame(jobfrm, chomp::krak::At(jobfrm.lRow/2+1,jobfrm.lCol-2),
      chomp::krak::At(jobfrm.lRow-5,jobfrm.lCol), jobfrm.bgColor, jobfrm.fgColor);
}

inline chomp::krak::del_ins_job::del_ins_job(const chomp::krak::frstring &t,
   chomp::krak::list &l,const chomp::krak::Frame &f): chomp::krak::sel_job(t,l,f)
{
   furnish();
   lastshown = chomp::min(length,maxshown)-1;
   search_permitted=1;
}

inline int chomp::krak::del_ins_job::type()
{
   return DEL_INS_JOB;
}

//############################  class ed_rec_job ###############################

inline void chomp::krak::ed_rec_job::furnish(void)
{
   window = chomp::krak::Frame(jobfrm, chomp::krak::At(3,1),
      chomp::krak::At(jobfrm.lRow-1,edit_field_start-2),WHITE,BLACK);
   maxshown=window.NoRow();
   editfrm = chomp::krak::Frame(jobfrm, chomp::krak::At(3,edit_field_start),
      chomp::krak::At(jobfrm.lRow-1,jobfrm.lCol-3),WHITE,BLACK);
   upFrm = chomp::krak::Frame(jobfrm, chomp::krak::At(3,jobfrm.lCol-2),
      chomp::krak::At(jobfrm.lRow/2,jobfrm.lCol), jobfrm.bgColor, jobfrm.fgColor);
   downFrm = chomp::krak::Frame(jobfrm, chomp::krak::At(jobfrm.lRow/2+1,jobfrm.lCol-2),
      chomp::krak::At(jobfrm.lRow-1,jobfrm.lCol), jobfrm.bgColor, jobfrm.fgColor);
   edit_field_length=jobfrm.lCol-1-edit_field_start;
}

inline chomp::krak::ed_rec_job::ed_rec_job(const chomp::krak::frstring &t,
   chomp::krak::record &r,const chomp::krak::Frame &f) : chomp::krak::sel_job(t,r,f)
{
   set_edit_field_start(40);
   furnish();
   lastshown = chomp::min(length,maxshown)-1;
   search_permitted=1;
}

inline void chomp::krak::ed_rec_job::set_edit_field_start(int percent)
{
   edit_field_start=percent*jobfrm.lCol/100;
}

inline int chomp::krak::ed_rec_job::type()
{
   return ED_REC_JOB;
}

//#############################################################################

#if defined(KRAK_ERRORS)
namespace chomp{
namespace krak{
   extern chomp::krak::view_job *view_cerr;
}}
   #define cerr (* view_cerr)
   #define exit  waitBt();exit
#endif

// Obecnosc tego wzorca powoduje, ze gcc wywala sie pod LINUXem,
// jezeli jest wlaczona opcja kompilacji "-fexternal-templates".
namespace chomp{
namespace krak{
template<class OBJECT>
   chomp::krak::view_job &operator<<(chomp::krak::view_job &s, OBJECT &o){
      chomp::krak::frstring f;
      f << o;
      s << f;
      return s;
   }
}}

#endif

/// @}
