/// @addtogroup krak
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file primitiv.h
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _krak_
#define _krak_

#include <cstdarg>
#include "capd/krak/hardware.h"
#include "capd/krak/kernel.h"

#include "capd/krak/frstring.h"

/* Basic drawing routines */
namespace chomp{
namespace krak{
   void waitBt(void);
   void waitMouseBt(void);
   void errorExit(const char *fmt,...);
   void warning(const char *fmt,...);
   void moveTo(double x,double y);
   void lineTo(double x,double y);
   void plotDot(double x,double y);
   void Segment(int i0,int j0,int i1,int j1);
   void Rctngl(int lti,int ltj,int rbi,int rbj);
   void DrawRct(chomp::krak::Rct *r);
   void Square(int i,int j,int size);
   void Cross(int i,int j,int size);
   void Diamond(int i,int j,int size);
   void Xcross(int i,int j,int size);
   int linear(int x,int x0,int y0,int x1,int y1);
   void SetRct(chomp::krak::Rct *r,int lti,int ltj,int rbi,int rbj);

/* constants definitions */

//#define NO_KEY 0xff


//#include "capd/krak/pattern.h"

typedef DINT HG * DINT_P;
typedef char HG * CHAR_P;
typedef unsigned char HG * BYTE_P;

class Frame;

extern int fontHght,fontWdth;
extern chomp::krak::Frame *cFrm;
extern chomp::krak::Frame rootFrame;

extern chomp::krak::Frame *rootFrm;
extern char colorname[24][24];
extern int gray[];
extern int c_bgcol,c_fgcol;
}}

/* macro definitions */

#define openGW openGraphics
#define closeGW closeGraphics
#define dscrGW(a,b,c,d) dscrFrm(rootFrm,a,b,c,d)
#define setRct(r,a,b,c,d) SetRct(r,sc_i(a),sc_j(d),sc_i(c),sc_j(b))
#define drawRct(r) DrawRct(r)
#define rctngl(a,b,c,d) Rctngl(sc_i(a),sc_j(d),sc_i(c),sc_j(b))
#define segment(a,b,c,d) Segment(sc_i(a),sc_j(b),sc_i(c),sc_j(d))
#define cross(x,y,s) Cross(sc_i(x),sc_j(y),s)
#define crcl(x,y,s) Crcl(sc_i(x),sc_j(y),s)
#define square(x,y,s) Square(sc_i(x),sc_j(y),s)
#define diamond(x,y,s) Diamond(sc_i(x),sc_j(y),s)
#define xcross(x,y,s) Xcross(sc_i(x),sc_j(y),s)
#define PxlInFrm(f,p) ((p)->i>=(f)->lti && (p)->i<(f)->rbi && (p)->j>=(f)->ltj && (p)->j< (f)->rbj)
#define openFrm(a,b,c,d) openRelFrm(rootFrm,a,b,c,d)
#define opdsFrm(a,b,c,d,e,f,g,h) opdsRelFrm(rootFrm,a,b,c,d,e,f,g,h)

/* functions declarations */
namespace chomp{
namespace krak{
   unsigned long EstimFreeMem(void);
}}
#ifndef MAC
namespace chomp{
namespace krak{
   double Clock();
}}
#endif

namespace chomp{
namespace krak{
   extern void (*restore_window)(void);

   double tr_cubic(double t);
   void set_col(void);
   void set_pat(void);
   void free_pat(void);

   void MarkPt3D(chomp::krak::Frame *frm, double u, double v, double rb_col);

   void closeGraphics(void);
   void DrawTxtg(chomp::krak::Frame *frm,char *buf);
   void DrawTxtc(chomp::krak::Frame *frm,char *buf);
   void Rctngl_at(chomp::krak::Frame *frm,int lti,int ltj,int rbi,int rbj);
   void outchar(chomp::krak::Frame *frm,int row,int col,char * ch);
   void gprintf(chomp::krak::Frame *frm,const char *fmt,...);
   void gprintf_at(chomp::krak::Frame *frm,int row,int col,const char *fmt,...);
   void gcprintf(char *fmt,...);
   void gcprintf_at(int row,int col,char *fmt,...);
   void vgprintf_at(chomp::krak::Frame *frm,int row,int col,const char *fmt,std::va_list args);
   void drawFrm(chomp::krak::Frame *frmPtr);
   void scaleFrm(chomp::krak::Frame *frmPtr);
   void clrFrm(chomp::krak::Frame *frmPtr);
   void selFrm(chomp::krak::Frame *frmPtr);
   void initFrm(chomp::krak::Frame *frmPtr, int arglti,int argltj,int argrbi, int argrbj,
      int bgc=WHITE, int fgc=BLACK,
      int im = chomp::krak::fontHght/2, int jm = chomp::krak::fontWdth/2
   );
   void dscrFrm(chomp::krak::Frame *frm,double swx,double swy,double nex,double ney);
   chomp::krak::Frame *openRelFrm(chomp::krak::Frame *prntFrm,int lti,int ltj,int rbi,int rbj);
   chomp::krak::Frame *openTRelFrm(chomp::krak::Frame *prntFrm,int ,int ,int ,int );
   chomp::krak::Frame *opdsRelFrm(chomp::krak::Frame *prntFrm,int lti,int ltj,int rbi,int rbj,
      double swx,double swy,double nex,double ney);
   void clseFrm(chomp::krak::Frame *frm);
   void Clear(chomp::krak::Frame *frm, int color);
   void openGraphics(int hrs,int vrs,int bgcol,int fgcol,int ltx=0,int lty=0);
   void redrawCtrlFrm(void);
   void openCtrlFrm(int lti,int ltj,int rbi,int rbj);
   int sc_i(double x);
   int sc_j(double y);
   double x_sc(int i);
   double y_sc(int j);
   int MouseInFrm(chomp::krak::Frame *f);
   int in_cFrm(double x,double y);
   int inscr(int i,int j);
   int inchar(void);
   void getline(chomp::krak::Frame *frm,int row,int col,char *txt);
   void testExit(void);
   void testPause(void);
   double time_meter(void);
   long tick_meter(void);
   void bp(void);
   void stop(void);
   void delay(double sleepTime);
   void struct_copy(void *from,void *to,int size);
   double DREALmin(int count,double x);
   void openDbgFrm(int lti,int ltj,int rbi,int rbj);
   void vv(int i,double r);
   void ww(int i,double r);
   void qq(int b);
   void pp(void);
   void default_restore_window(void);

#ifdef DOS16
   void myfread(CHAR_P ptr,int size,int n,FILE *f);
   void myfwrite(CHAR_P ptr,int size,int n,FILE *f);
#endif

   void pause(int i,int j, chomp::krak::frstring s);
}} // the end of the namespace chomp::krak

#endif

/// @}
