/// @addtogroup interval
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file rounding-amd64.h
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _ROUNDING_AMD64_H_
#define _ROUNDING_AMD64_H_

// Note: we use the 'w' instructions as we only need to modify the low 16 bits
// of the register
#define ROUND_FUNC_BODY(mode) \
  unsigned int mxcsr; \
  asm volatile ( \
    "stmxcsr %0\n\t" \
    "andw $0x9fff, %1\n\t" \
    "orw $0x" mode ", %1\n\t" \
    "ldmxcsr %0\n\t" \
    : "=m" (mxcsr) \
    : "m" (mxcsr) \
  ); \

inline void round_nearest()
{
  ROUND_FUNC_BODY("0000");
}

inline void round_down()
{
  ROUND_FUNC_BODY("2000");
}

inline void round_up()
{
  ROUND_FUNC_BODY("4000");
}

inline void round_cut()
{
  ROUND_FUNC_BODY("6000");
}

inline int AMD64_getMXCSR()
{
  int mxcsr;
  asm volatile (
    "stmxcsr %0\n\t"
    : "=m" (mxcsr)
  );
  return mxcsr;
}

#undef ROUND_FUNC_BODY

#endif /*_ROUNDING_AMD64_H_*/
/// @}
