/// @addtogroup vectalg
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file normintf.h
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include <string>

#include "capd/vectalg/vectorintf.h"
#include "capd/vectalg/matrixintf.h"

#ifndef _NormTemplate
#define _NormTemplate

template<typename VectorType,typename MatrixType>
class Norm{
public:
   typedef typename VectorType::ScalarType ScalarType;

   virtual ScalarType operator()(const VectorType &iv) const = 0;
   virtual ScalarType operator()(const MatrixType &iv) const = 0;
   virtual std::string show(void) const = 0;
   virtual Norm *clone(void) const = 0;
   virtual ~Norm() {}
};

template<typename VectorType,typename MatrixType>
class EuclNorm : public Norm<VectorType,MatrixType>{
public:
   typedef typename VectorType::ScalarType ScalarType;

   virtual ScalarType operator()(const VectorType &iv) const;
   virtual ScalarType operator()(const MatrixType &iv) const;
   virtual std::string show(void) const;
   Norm<VectorType,MatrixType> * clone(void) const;
};

template<typename VectorType,typename MatrixType>
class MaxNorm : public Norm<VectorType,MatrixType>{
public:
   typedef typename VectorType::ScalarType ScalarType;

   virtual ScalarType operator()(const VectorType &iv) const;
   virtual ScalarType operator()(const MatrixType &iv) const;
   virtual std::string show(void) const;
   Norm<VectorType,MatrixType> * clone(void) const;
};

template<typename VectorType,typename MatrixType>
class SumNorm : public Norm<VectorType,MatrixType>{
public:
   typedef typename VectorType::ScalarType ScalarType;

   virtual ScalarType operator()(const VectorType &iv) const;
   virtual ScalarType operator()(const MatrixType &iv) const;
   virtual std::string show(void) const;
   Norm<VectorType,MatrixType> * clone(void) const;
};

template<typename VectorType,typename MatrixType>
class EuclLNorm : public Norm<VectorType,MatrixType>{
public:
   typedef typename VectorType::ScalarType ScalarType;

   virtual ScalarType operator()(const VectorType &iv) const;
   virtual ScalarType operator()(const MatrixType &iv) const;
   virtual std::string show(void) const;
   Norm<VectorType,MatrixType> * clone(void) const;
};

template<typename VectorType,typename MatrixType>
class MaxLNorm : public Norm<VectorType,MatrixType>{
public:
   typedef typename VectorType::ScalarType ScalarType;

   virtual ScalarType operator()(const VectorType &iv) const;
   virtual ScalarType operator()(const MatrixType &iv) const;
   virtual std::string show(void) const;
   Norm<VectorType,MatrixType> * clone(void) const;
};

template<typename VectorType,typename MatrixType>
class SumLNorm : public Norm<VectorType,MatrixType>{
public:
   typedef typename VectorType::ScalarType ScalarType;

   virtual ScalarType operator()(const VectorType &iv) const;
   virtual ScalarType operator()(const MatrixType &iv) const;
   virtual std::string show(void) const;
   Norm<VectorType,MatrixType> * clone(void) const;
};

#endif // #define _NormTemplate

/// @}
