/// @addtogroup vectalg

/// @{

 

/////////////////////////////////////////////////////////////////////////////

/// @file subVectorIntf.h

///

/// @author Tomasz Kapela

/////////////////////////////////////////////////////////////////////////////



// Copyright (C) 2007 by the CAPD Group.

//

// This file is part of the CAPD library.  This library is free software;

// you can redistribute it and/or modify it under the terms of the GNU

// General Public License as published by the Free Software Foundation;

// either version 2 of the License, or (at your option) any later version.

//

// This library is distributed in the hope that it will be useful,

// but WITHOUT ANY WARRANTY; without even the implied warranty of

// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

// GNU General Public License for more details.

//

// You should have received a copy of the GNU General Public License along

// with this software; see the file "license.txt".  If not, write to the

// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,

// MA 02111-1307, USA.



#ifndef _SUBVECTOR_INTF_H_

#define _SUBVECTOR_INTF_H_ 1



/// Intersection of src with target[start..stop] is returned in target

template<typename VectorType1, typename VectorType2>

void intersectionWithSubvector(const VectorType1 & src, VectorType2 & target, 

                               int start, int stop);



/// it copies src into target[start..stop]

template<typename VectorType1, typename VectorType2>

void copyToSubvector(const VectorType1 & src,  VectorType2 & target, 

                     int start, int stop);



/// it copies src[start..stop] into target

template<typename VectorType1, typename VectorType2>

void copyFromSubvector(const VectorType1& src, VectorType2& target, 

                       int start, int stop);



/// intersection of src with target[startRow..stopRow][startCol..stopCol] is returned in target

template<typename MatrixType1, typename MatrixType2>

void intersectionWithSubmatrix( const MatrixType1 & src, 

                                MatrixType2 & target, 

                                int startRow, int stopRow, 

                                int startCol, int stopCol);



/// it copies src into target[startRow..stopRow][startCol..stopCol]

template<typename MatrixType1, typename MatrixType2>

void copyToSubmatrix(const MatrixType1 & src, 

                     MatrixType2 & target, 

                     int startRow, int stopRow, 

                     int startCol, int stopCol);



/// it copies src[startRow..stopRow][startCol..stopCol] into target

template<typename MatrixType1, typename MatrixType2>

void copyFromSubmatrix(const MatrixType1 & src, 

                       MatrixType2 & target, 

                       int startRow, int stopRow, 

                       int startCol, int stopCol);



#endif //_SUBVECTOR_INTF_H_


/// @}
