/// @addtogroup matrixAlgorithms
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file floatMatrixAlgorithms.cpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include "capd/capd/minmax.h"
#include "capd/vectalg/vectalglib.h"

template void orthonormalize<matrix>(matrix&);
template void orthonormalize<intervalMatrix>(intervalMatrix&);

template void QR_decompose<matrix>(const matrix&,matrix&,matrix&);
template void QR_decompose<intervalMatrix>(const intervalMatrix&,intervalMatrix&,intervalMatrix&);

template int symMatrixDiagonalize<matrix>(const matrix&,matrix&,double);
template int symMatrixDiagonalize<intervalMatrix>(const intervalMatrix&,intervalMatrix&,interval);

template double spectralRadiusOfSymMatrix<matrix>(const matrix&,double);
template interval spectralRadiusOfSymMatrix<intervalMatrix>(const intervalMatrix&,interval);

template double maxEigenValueOfSymMatrix<matrix>(const matrix&,double);
template interval maxEigenValueOfSymMatrix<intervalMatrix>(const intervalMatrix&,interval);

template vector gauss<matrix,vector>(matrix, vector);
template intervalVector gauss<intervalMatrix,intervalVector>(intervalMatrix,intervalVector);

template void croutDecomposition<matrix>(const matrix&,matrix&,matrix&);
template void croutDecomposition<intervalMatrix>(const intervalMatrix&,intervalMatrix&,intervalMatrix&);

template matrix invLowerTriangleMatrix<matrix>(const matrix&);
template intervalMatrix invLowerTriangleMatrix<intervalMatrix>(const intervalMatrix&);

template matrix invUpperTriangleMatrix<matrix>(const matrix&);
template intervalMatrix invUpperTriangleMatrix<intervalMatrix>(const intervalMatrix&);

template matrix inverseMatrix<matrix>(const matrix&);
template intervalMatrix inverseMatrix<intervalMatrix>(const intervalMatrix&);

template matrix gaussInverseMatrix<matrix>(const matrix);
template intervalMatrix gaussInverseMatrix<intervalMatrix>(const intervalMatrix);

/// @}
